/*******************************************************************************
 *******************************************************************************
 ** High level KERNAL API routines                                            **
 *******************************************************************************
 *******************************************************************************/

/*******************************************************************************
 * Open file                                                                   *
 *                                                                             *
 * Using LA, FA, SA and FNADR & FNLEN open a new logical file.                 *
 *******************************************************************************/

	subroutine

zopen:
	ldx	la		; get logical file
	bne	.find		; valid, continue

	jmp	fe_ntinp	; else do not input file error and return

.find:
	jsr	fndflno		; find file table entry
	bne	.check_count	; not found, continue

	jmp	fe_alopn	; else do file already open error and return

.check_count:
	ldx	ldtnd		; get open file count
	cpx	#10		; compare with max files
	bcc	.add		; still space, continue

	jmp	fe_2mnyf	; else do too many files error and return

.add:
	inc	ldtnd		; increment open file count
	lda	la		; get logical file
	sta	lat,x		; save to logical file table
	lda	sa		; get secondary address
	ora	#$60		; OR with the OPEN CHANNEL command
	sta	sa		; set secondary address
	sta	sat,x		; save to secondary address table
	lda	fa		; get device number
	sta	fat,x		; save to device number table
	cmp	#8		; compare with first disk device
	bcc	.k_open		; not a serial bus disk device

	lda	sa		; get secondary address
	ora	#$f0		; OR with OPEN command
	tay			; copy to .Y
	jsr	ser_str		; send secondary address and file name
	rts

.k_open:
	tax			; restore flags
	jmp	$f437		; use normal KERNAL routine


/*******************************************************************************
 * Close file                                                                  *
 *                                                                             *
 * Close logical file handle in .A.                                            *
 *******************************************************************************/

	subroutine

zclose:
	jsr	$f3d4		; find file index for .A
	bne	.out		; if not found the file was closed so just flag ok

	jsr	setflch		; set file details from table,.X
	txa			; copy file index to .A
	pha			; save file index
	lda	fa		; get device number
	cmp	#8		; compare with first disk device
	bcc	.k_close	; not a serial bus disk device

	lda	sa		; get secondary address
	ora	#$e0		; OR with CLOSE command
	tay			; copy to .Y
	jsr	ser_close	; close serial channel
	jmp	$f3b1		; remove from file table

.k_close:
	jmp	$f356		; use normal KERNAL routine

.out:
	clc			; flag ok
	rts


/*******************************************************************************
 * Set logical file handle as output                                           *
 *                                                                             *
 * Send all output to the device associated with file handle in .X.            *
 *******************************************************************************/

	subroutine

zchkout:
	jsr	fndflno		; find file table entry
	bne	.no_file	; not found, file not open

	jsr	setflch		; set file details from table,.X
	lda	fa		; get device number
	cmp	#8		; compare with first disk device
	bcc	.k_chkout	; not a serial bus disk device

	jsr	listen		; command device to LISTEN
	lda	sa		; get secondary address
	jsr	second		; target message to secondary address
	bit	status		; test I/O status byte
	bmi	.no_dev		; no device, error

	lda	fa		; get device number
	jmp	$f32e		; set output device

.k_chkout:
	jmp	$f314		; use normal KERNAL routine

.no_file:
	jmp	fe_ntopn	; file not open error and return

.no_dev:
	jmp	fe_dvntp	; device not present error and return


/*******************************************************************************
 * Set logical file handle as input                                            *
 *                                                                             *
 * Get all input from the device associated with file handle in .X.            *
 *******************************************************************************/

zchkin:
	jsr	fndflno		; find file table entry
	bne	.no_file	; not found, file not open

	jsr	setflch		; set file details from table,.X
	lda	fa		; get device number
	cmp	#8		; compare with first disk device
	bcc	.k_chkin	; not a serial bus disk device

	jsr	talk		; command device to TALK
	lda	sa		; get secondary address
	jsr	tksa		; send secondary address after TALK
	bit	status		; test I/O status byte
	bmi	.no_dev		; no device, error

	lda	fa		; get device number
	jmp	$f2ec		; set input device

.k_chkin:
	jmp	$f2d2		; use normal KERNAL routine


/*******************************************************************************
 * Send a byte to the output device                                            *
 *                                                                             *
 * Transmit the byte in .A to the default output device.                       *
 *******************************************************************************/

	subroutine

zchrout:
	pha			; save the byte to send
	lda	dflto		; get output device number
	cmp	#8		; compare with first disk device
	bcc	.k_chrout	; not a serial bus disk device

	stx	xsav		; save .X
	pla			; restore byte to send
	jsr	ciout		; send byte to listening serial bus device
	ldx	xsav		; restore .X
	rts

.k_chrout:
	jmp	$f27d		; use normal KERNAL routine


/*******************************************************************************
 * Get a byte from the input device                                            *
 *                                                                             *
 * Receive a byte from the default input device and store it in .A.            *
 *******************************************************************************/

	subroutine

zgetin:
	lda	dfltn		; get input device number
	cmp	#8		; compare with first disk device
	bcc	.k_getin	; not a serial bus disk device

zchrin:
	lda	dfltn		; get input device number
	cmp	#8		; compare with first disk device
	bcc	.k_chrin	; not a serial bus disk device

	stx	xsav		; save .X
	jsr	acptr		; get byte from talking serial bus device
	ldx	xsav		; restore .X
	lda	firt		; reload received byte (for flags)
	rts

.k_getin:
	jmp	fgetin		; use normal KERNAL routine

.k_chrin:
	jmp	fchrin		; use normal KERNAL routine


/*******************************************************************************
 * Close all file handles                                                      *
 *******************************************************************************/

	subroutine

zclall:
	lda	#0
	sta	ldtnd		; clear open file count


/*******************************************************************************
 * Restore input and output devices to defaults                                *
 *******************************************************************************/

zclrchn:
	ldx	#3		; last internal device
	cpx	dflto		; compare with output device
	bcs	.check_in	; internal device, check input

	jsr	unlsn		; command serial bus to UNLISTEN

.check_in:
	cpx	dfltn		; compare with input device
	bcs	.set_dflt	; internal device, set default devices

	jsr	untlk		; command serial bus to UNTALK

.set_dflt:
	ldx	#3		; screen device
	jmp	$f403		; set default devices
