/*******************************************************************************
 *******************************************************************************
 ** Save to serial bus device						      **
 *******************************************************************************
 *******************************************************************************/

	subroutine

.not_disk:
	jmp	fsave2		; use normal KERNAL routine

zsave:
	lda	fa		; get device number
	cmp	#8		; compare with first disk device
	bcc	.not_disk	; not a serial bus disk device

	lda	#0		; clear .A
	sta	status		; clear I/O status byte
	ldy	#$f1		; OPEN command
	jsr	ser_str		; send secondary address and filename
	ifnconst OPT_PRINT_ADDR
	jsr	saving		; print "SAVING <file name>"
	endif
	lda	fa		; get device number
	jsr	listen		; command device to LISTEN
	lda	#$61		; file write
	jsr	second		; send secondary address after LISTEN
	jsr	rd300		; copy start address to current address
	ifconst OPT_CART_SA
	lda	sa		; get secondary address
	cmp	#2		; compare with ROM image
	beq	.payload	; ROM image, skip writing start address
	endif

	lda	stal		; get start address low byte
	jsr	ciout		; output byte on serial bus
	lda	stah		; get start address high byte
	jsr	ciout		; output byte on serial bus
	lda	status		; get I/O status byte
	bne	.error		; write to file rejected, flag error

.payload:
	ifconst OPT_PRINT_ADDR
	ldx	#sal		; address location
	jsr	print_from_addr	; display "FROM $xxxx"
	endif
	ldy	#0		; clear index
.loop:
	jsr	vprty		; check current address against end address
	bcs	.done		; reached end, close channel

	lda	(sal),y		; fetch byte
	jsr	ciout		; output byte on serial bus
	jsr	stop		; scan stop key
	beq	.stop		; [STOP] pressed close and flag stop

	jsr	wrt62		; increment current address
	bne	.loop		; branch always

.error:
	jsr	.close		; close channel
	jmp	fe_ntfnd	; return file not found

.stop:
	jsr	.close		; close channel
	lda	#0		; STOP pressed
	sec			; flag error
	rts

.done:
	ifconst OPT_PRINT_ADDR
	ldx	#sal		; address location
	jsr	print_to_addr	; display "TO $xxxx"
	endif

.close:
	jsr	unlsn		; command serial bus to UNLISTEN
	ldy	#$e1		; CLOSE write channel
	jsr	ser_close	; close channel
	ifconst OPT_PRINT_ADDR
	jsr	print_cr
	endif
	clc			; flag ok
	rts
