	processor	6502

val	= $ac
ADDR	= $f7
EADDR	= $f9
PTR	= $fb
TMP	= $fd

setlfs  = $ffba
setnam  = $ffbd

	; VICE debug cart I/O address
dbg	= $910f


	org $a000

	subroutine

	jmp	init_and_test
	jmp	do_test
.loop:	jsr	do_test
	beq	.loop
	rts

init_and_test:
	nop

do_test:
	ifconst	DBG_UART
	jsr	RSINIT
	endif

	ldx	#8
	ldy	#2
	jsr	setlfs
	ldx	#<fname
	ldy	#>fname
	lda	#end-fname
	jsr	setnam
	lda	#'S
	ifconst DBG_VICE
	sta	dbg
	endif
	ifconst DBG_UART
	jsr	RSSEND
	endif
	jsr	read_file
	bcs	lerr
	lda	#'L
	ifconst DBG_VICE
	sta	dbg
	endif
	ifconst DBG_UART
	jsr	RSSEND
	endif
	txa
	bne	szerr
	cpy	#$80
	bne	szerr
	lda	#$12
	sta	ADDR+1
	lda	#$80
	sta	EADDR+1
	ldx	#val
	jsr	CRC32
	lda	val
	eor	good_crc
	bne	cerr
	lda	val+1
	eor	good_crc+1
	bne	cerr
	lda	val+2
	eor	good_crc+2
	bne	cerr
	lda	val+3
	eor	good_crc+3
	bne	cerr
	lda	#0
	ifconst DBG_VICE
	sta	dbg
	endif
	ifconst DBG_UART
	jsr	RSSEND
	endif
	lda	#$1d
	sta	$900f
	rts

cerr:				; crc error
	lda	#'C
	dc.b	$2c
szerr:				; length error
	lda	#'Z
	dc.b	$2c
lerr:				; load error
	lda	#'E
	ifconst DBG_VICE
	sta	dbg
	endif
	ifconst DBG_UART
	jsr	RSSEND
	endif
	lda	#$1a
	sta	$900f
	lda	#0
	rts


	subroutine

read_file:
	lda	#0
	sta	status
	ldy	#$f0		; OPEN command
	jsr	ser_str		; send secondary address and filename
	lda	fa		; get device number
	jsr	talk		; command serial bus device to TALK
	lda	#$60		; file read
	jsr	tksa		; send secondary address after TALK
	lda	sa		; get secondary address

	jsr	acptr		; get byte from file
	sta	eal		; set current address low byte
	lda	status		; get I/O status byte
	bne	.not_found	; timed out, do file not found and return

	jsr	acptr		; get byte from file
	sta	eah		; set current address high byte

	ldy	#0		; clear index

.loop:
	jsr	acptr		; get byte from file
	sta	(eal),y		; store byte
.inc_addr:
	inc	eal		; increment current address low byte
	bne	.test_eof	; no overflow, test for EOF

	inc	eah		; increment current address high byte

.test_eof:
	bit	status		; test I/O status byte
	bvc	.loop		; not EOF, continue

	jsr	untlk		; command serial bus to UNTALK
	ldy	#$e0		; CLOSE read command
	jsr	ser_close	; close channel
	ldx	eal		; get end address low byte
	ldy	eah		; get end address high byte
	clc			; flag ok
	rts

check_stop:
.not_found:
	jmp	fe_ntfnd	; return file not found

good_crc:
	dc.l	$ff46badc

fname:
	dc.b	"TEST-FILE"
end:

	include "crc.asm"

	align	$100

	include "../kernal-defs.asm"
	include "../core.asm"

	ifconst	DBG_UART
	include "uart.asm"
	endif
